import copy
import logging
from threading import Semaphore, Thread

import circum.endpoint

import click


logger = logging.getLogger(__name__)
tracking_semaphore = None
tracking_info = {"objects": []}
vector_info = []
updated = False


def _get_targets():
    raise NotImplementedError()
    return []


def _update_thread(kinect_update_fn, kinect):
    global tracking_info
    global vector_info
    global updated

    while True:

        tracking_semaphore.acquire()

        tracks, new = kinect_update_fn(kinect)
        if new:
            updated = True
            tracking_info["objects"] = tracks
            logger.debug(f'got: {tracking_info}')

        tracking_semaphore.release()
        # time.sleep(update_interval)


def run_kinect(kinect_args: {}) -> {}:
    global updated
    ret = None
    tracking_semaphore.acquire()
    if updated:
        logger.debug(f"sending {tracking_info}")
        ret = copy.deepcopy(tracking_info)
        updated = False
    tracking_semaphore.release()
    return ret


def _kinect(ctx,
            version: str):
    global tracking_semaphore
    tracking_semaphore = Semaphore()

    if version == '1':
        try:
            from circum_kinect.pykinect import init_pykinect, pykinect_update_thread
        except Exception as e:
            logger.error("Unable to load pykinect, is the package installed? Try `pip3 install pykinect`.")
            raise e
        kinect = init_pykinect()
        kinect_update_fn = pykinect_update_thread
    elif version == '2':
        try:
            from circum_kinect.pykinect2 import init_pykinect2, pykinect2_update_thread
        except Exception as e:
            logger.error("Unable to load pykinect2, is the package installed? Try `pip3 install pykinect2`.")
            raise e
        kinect = init_pykinect2()
        kinect_update_fn = pykinect2_update_thread
    else:
        raise ValueError(f"Unsupported version {version}. Only versions 1 and 2 are supported")

    tracker_thread = Thread(target=_update_thread, args=[kinect_update_fn, kinect])
    tracker_thread.daemon = True
    tracker_thread.start()
    circum.endpoint.start_endpoint(ctx, "kinect", run_kinect)


@click.command()
@click.option('--version',
              required=False,
              type=click.Choice(['1', '2'], case_sensitive=False),
              default='2',
              help='PyKinect version to use.')
@click.pass_context
def kinect(ctx,
           version: str):
    _kinect(ctx, version)



import logging

from pykinect2 import PyKinectRuntime
from pykinect2.PyKinectV2 import FrameSourceTypes_Body, JointType_SpineShoulder, TrackingState_NotTracked


logger = logging.getLogger(__name__)


def init_pykinect2():
    kinect2 = PyKinectRuntime.PyKinectRuntime(FrameSourceTypes_Body)
    return kinect2


def pykinect2_update_thread(kinect2):
    if kinect2.has_new_body_frame():
        logger.debug(f"new body frame with {kinect2.max_body_count} bodies!")
        targets = kinect2.get_last_body_frame()
    else:
        targets = None

    tracking = []
    updated = False

    if targets and targets is not None:
        updated = True
        for i in range(0, kinect2.max_body_count):
            target = targets.bodies[i]
            if not target.is_tracked:
                continue

            joints = target.joints

            # use the Spine Shoulder joint as a shortcut for center of mass
            spine_shoulder = joints[JointType_SpineShoulder]

            if spine_shoulder.TrackingState == TrackingState_NotTracked:
                continue

            # TODO: add sensor specific details (joints, joint normals, hand positions, face tracking, etc.)
            track = {
                "x": spine_shoulder.Position.x,
                "y": spine_shoulder.Position.y,
                "z": spine_shoulder.Position.z
            }

            tracking.append(track)

    return tracking, updated



from pykinect import nui
from pykinect.nui import JointId


def init_pykinect():
    kinect = nui.Runtime()
    kinect.skeleton_engine.enabled = True
    return kinect


def pykinect_update_thread(kinect):
    skeletons = kinect.skeleton_engine.get_next_frame().SkeletonData

    # TODO: add sensor specific information (joints)
    tracking = [
        {
            "x": data.SkeletonPositions[JointId.ShoulderCenter][0],
            "y": data.SkeletonPositions[JointId.ShoulderCenter][1],
            "z": data.SkeletonPositions[JointId.ShoulderCenter][2]
        }
        for data in skeletons
    ]

    return tracking, True



from setuptools import setup, find_packages
from os import path


this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


test_requirements = [
    'pytest',
    'pytest-cov',
]

setup(
    name='circum_kinect',
    version_format='{tag}',
    author="Lane Haury",
    author_email="lane@lumineerlabs.com",
    description="Kinect sensor plugin for circum.",
    license="MIT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/LumineerLabs/circum-kinect",
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        'circum',
        'click',
    ],
    setup_requires=[
        'setuptools',
        'setuptools-git-version',
    ],
    tests_require=test_requirements,
    entry_points={
        'circum.sensors': [
            'kinect=circum_kinect.kinect:kinect'
        ]
    },
    extras_require={
        'lint': [
            'flake8',
            'flake8-import-order',
            'flake8-builtins',
            'flake8-comprehensions',
            'flake8-bandit',
            'flake8-bugbear',
        ],
        'pykinect': [
            'pykinect @ https://github.com/LumineerLabs/PTVS/archive/9c9b02416e46a8661fd324a5d66f9b32dff0b55a.zip'
        ],
        'pykinect2': [
            'pykinect2 @ https://github.com/LumineerLabs/PyKinect2/archive/81f9a55d5ceeec342dc0c95b0f8084b5816aca08.zip',
        ],
        'test': test_requirements,
    }
)
